package com.mortals.xhx.daemon.applicationservice;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.springcloud.service.IApplicationStartedService;
import com.mortals.framework.util.DateUtils;
import com.mortals.framework.util.ThreadPool;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.device.service.DeviceService;
import com.mortals.xhx.module.face.model.FaceGroupEntity;
import com.mortals.xhx.module.face.model.FaceGroupQuery;
import com.mortals.xhx.module.face.service.FaceGroupService;
import com.mortals.xhx.module.face.service.FacePlanService;
import com.mortals.xhx.module.hik.event.model.req.sub.EventSubReq;
import com.mortals.xhx.module.hik.event.model.rsp.EventInfo;
import com.mortals.xhx.module.hik.event.service.IHikEventService;
import com.mortals.xhx.module.hik.face.model.req.group.FaceGroupReq;
import com.mortals.xhx.module.hik.face.model.req.plan.PlanRecognListReq;
import com.mortals.xhx.module.hik.face.model.req.plan.PlanRecognReq;
import com.mortals.xhx.module.hik.face.model.rsp.group.FaceGroupDataInfo;
import com.mortals.xhx.module.hik.face.model.rsp.plan.PlanRecognInfo;
import com.mortals.xhx.module.hik.face.service.IHikFaceService;
import com.mortals.xhx.module.hik.face.service.IHikPlanService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.ParamKey.PARAM_FACE_GROUP;
import static com.mortals.xhx.common.key.ParamKey.PARAM_FACE_GROUP_REGISTER;


@Component
@Slf4j
public class SubEventStartedService implements IApplicationStartedService {

    @Value("${hik.callback:''}")
    private String callback;

    @Autowired
    private IHikEventService hikEventService;
    @Autowired
    private IHikFaceService hikFaceService;
    @Autowired
    private IHikPlanService hikPlanService;
    @Autowired
    private FaceGroupService faceGroupService;
    @Autowired
    private FacePlanService facePlanService;
    @Autowired
    private DeviceService deviceService;


    @Override
    public void start() {
        ThreadPool.getInstance().init(20);
        log.info("开始服务..[事件订阅服务]");
        String eventtypes = GlobalSysInfo.getParamValue(Constant.PARAM_EVENTTYPES, "131614,131659,1644175361,1644171265");
        List<Integer> eventTypes = StrUtil.split(eventtypes, ",".charAt(0), -1, false, Integer::parseInt);

        EventSubReq eventSubReq = new EventSubReq();
        eventSubReq.setEventTypes(eventTypes);
        eventSubReq.setEventDest(callback);
        Rest<String> rest = hikEventService.subEvent(eventSubReq);
        log.info("sub events resp==>{}", JSON.toJSONString(rest));
        Rest<List<EventInfo>> events = hikEventService.getEvents();
        log.info("get events resp==>{}", JSON.toJSONString(events));

        //查询本地指定人脸分组 如果没有则添加 并更新

        String groupStr = GlobalSysInfo.getParamValue(PARAM_FACE_GROUP, PARAM_FACE_GROUP_REGISTER);
        List<String> groupList = StrUtil.split(groupStr, ",".charAt(0));

        for (String group : groupList) {
            FaceGroupEntity faceGroupEntity = faceGroupService.selectOne(new FaceGroupQuery().name(group));
            if (ObjectUtils.isEmpty(faceGroupEntity)) {
                log.info("创建人脸分组...");
                FaceGroupReq faceGroupReq = new FaceGroupReq();
                faceGroupReq.setName(group);
                faceGroupReq.setDescription(group);
                Rest<FaceGroupDataInfo> faceGroupRest = hikFaceService.faceGroupSingleAdd(faceGroupReq);
                if (faceGroupRest.getCode() == YesNoEnum.YES.getValue()) {
                    FaceGroupEntity groupEntity = new FaceGroupEntity();
                    groupEntity.initAttrValue();
                    groupEntity.setName(group);
                    groupEntity.setDescription(group);
                    groupEntity.setIndexCode(faceGroupRest.getData().getIndexCode());
                    groupEntity.setCreateTime(new Date());
                    groupEntity.setCreateUserId(1L);
                    groupEntity.setCreateUserName("admin");
                    faceGroupService.save(groupEntity);
                }
            }
        }

        //创建陌生人计划
        facePlanService.createStrangerPlanByDay();

        //删除重点人员计划
      /*  PlanRecognListReq planRecognBlackListReq = new PlanRecognListReq();
        Rest<List<PlanRecognInfo>> planRecognBlackRest = hikPlanService.findPlanRecognBlackList(planRecognBlackListReq);
        if (planRecognBlackRest.getCode() == YesNoEnum.YES.getValue()) {
            List<PlanRecognInfo> delPlanList = planRecognBlackRest.getData();
            if (!ObjectUtils.isEmpty(delPlanList)) {
                //删除计划
                PlanRecognReq planRecognBlackReq = new PlanRecognReq();
                planRecognBlackReq.setRecognitionResourceIndexCodes(delPlanList.stream().map(i -> i.getIndexCode()).collect(Collectors.toList()));
                Rest<Boolean> delRest = hikPlanService.planRecognBlackDel(planRecognBlackReq);
                if (delRest.getCode() == YesNoEnum.YES.getValue() && delRest.getData()) {
                    log.info("预约计划删除成功！");
                }
            }
        }*/


        //删除创建陌生人员识别计划，todo
        // String planBlackName = GlobalSysInfo.getParamValue(ParamKey.PARAM_FACE_PLAN_BLACK, "取号监控");

/*
        int threshold = GlobalSysInfo.getParamIntValue(ParamKey.PARAM_FACE_THRESHOLD, 80);
        FacePlanEntity facePlanEntity = facePlanService.selectOne(new FacePlanQuery().name(planBlackName));
        if (ObjectUtils.isEmpty(facePlanEntity)) {
            //初始化新增取号监控识别计划
            PlanRecognReq planRecognBlackReq = new PlanRecognReq();
            planRecognBlackReq.setName(planBlackName);
            planRecognBlackReq.setDescription(planBlackName);
            planRecognBlackReq.setThreshold(threshold);
            //设置人脸分组
            FaceGroupEntity faceGroupEntity = faceGroupService.selectOne(new FaceGroupQuery().name("注册群众"));
            List<String> faceGroupIndexCodes = new ArrayList<>();
            faceGroupIndexCodes.add(faceGroupEntity.getIndexCode());
            planRecognBlackReq.setFaceGroupIndexCodes(faceGroupIndexCodes);
            //设置监控点
            List<String> cameraIndexCodes = deviceService.find(new DeviceQuery()).stream().map(DeviceEntity::getDeviceCode).collect(Collectors.toList());
            planRecognBlackReq.setCameraIndexCodes(cameraIndexCodes);
            //设置识别方式
            planRecognBlackReq.setRecognitionResourceType(RecognitionResourceEnum.COMPARISON.getValue());

            //设置时间段 默认全天候
            Rest<String> planRecognBlackAddRest = hikPlanService.planRecognBlackAdd(planRecognBlackReq);
            if (planRecognBlackAddRest.getCode() == YesNoEnum.YES.getValue()) {
                FacePlanEntity planEntity = new FacePlanEntity();
                planEntity.initAttrValue();
                planEntity.setIndexCode(planRecognBlackAddRest.getData());
                planEntity.setName(planBlackName);
                planEntity.setDescription(planBlackName);
                planEntity.setCreateTime(new Date());
                planEntity.setCreateUserId(1L);
                planEntity.setCreateUserName("admin");
                facePlanService.save(planEntity);
            }

        }
*/

        //创建陌生人员识别计划


    }

    @Override
    public void stop() {
        log.info("停止服务..");
        ThreadPool.getInstance().close();
    }

    @Override
    public int getOrder() {
        return 10;
    }


}
